/* itemscombo.vala
 *
 * Copyright (C) 2010  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;
using Gee;

public class I4uc.GtkFrontend.ItemsCombo : Gtk.VBox
{
	private Label _items_label = new Label ("");
	private ListStore _items_store;
	private ComboBox _items_view;
	private Button _button = new Button ();
	private string _current_item_label;

	private ArrayList<string> _items = new ArrayList<string> ();

	private enum ItemsColumns
	{
		ITEM_ID,
		ITEM_LABEL,
		N_COLUMNS
	}
	
	public string label { set; get; }

	public string current_item
	{
		set
		{
			if (_items.size == 0)
				return;
			if (value == "")
			{
				_items_view.active = -1;
				return;
			}
			TreeIter iter;
			_items_store.get_iter_from_string (out iter, _items.index_of (value).to_string ());
			_items_store.get (iter, ItemsColumns.ITEM_LABEL, out _current_item_label);
			_items_view.set_active_iter (iter);
		}
		get
		{
			TreeIter iter;
			unowned string item = null;
			if (_items_view.get_active_iter (out iter))
				_items_store.get (iter, ItemsColumns.ITEM_ID, out item);
			return item;
		}
	}
	
	public string current_item_label { get { return _current_item_label; } }
	
	public string current_item_tooltip
	{
		set { _items_view.tooltip_markup = value; }
		get { return _items_view.tooltip_markup; }
	}
	
	public bool button_sensitive { set; get; }

	public signal void button_clicked ();

	public ItemsCombo (string label="")
	{
		init_widgets (label);
		pack_start (_items_label, false, false, 0);
		pack_start (_items_view, true, true, 0);
	}
	
	public ItemsCombo.with_button (string label, string stock_icon)
	{
		init_widgets (label);
		
		_button.add (new Image.from_stock (stock_icon, IconSize.MENU));
		
		var hbox = new HBox (false, 0);
		hbox.pack_start (_items_view, true, true, 0);
		hbox.pack_start (_button, false, false, 0);
		
		pack_start (_items_label, false, false, 0);
		pack_start (hbox, false, false, 0);
		
		//Bind properties
		bind_property ("button-sensitive", _button, "sensitive", BindingFlags.BIDIRECTIONAL);
		
		//signals
		_button.clicked.connect (() => this.button_clicked ());
	}

	public void add_item (string item_id, string? item_label=null)
	{
		if (item_id in _items)
			return;
		TreeIter iter;
		_items_store.append (out iter);
		_items_store.set (iter, ItemsColumns.ITEM_ID, item_id, ItemsColumns.ITEM_LABEL, item_label ?? item_id);
		_items.add (item_id);
	}
	
	public void change_item (string item_id, string new_item_id, string? new_item_label=null)
	{
		if (!(item_id in _items))
			return;
		var notify_prop = this.current_item == item_id;
		TreeIter iter;
		var index = _items.index_of (item_id);
		_items_store.get_iter_from_string (out iter, index.to_string ());
		_items_store.set (iter, ItemsColumns.ITEM_ID, new_item_id, ItemsColumns.ITEM_LABEL, new_item_label ?? new_item_id);
		_items.remove (item_id);
		_items.insert (index, new_item_id);
		if (notify_prop)
			notify_property ("current-item");
	}

	public void remove_item (string item_id)
	{
		if (!(item_id in _items))
			return;
		TreeIter iter;
		_items_store.get_iter_from_string (out iter, _items.index_of (item_id).to_string ());
		_items.remove (item_id);
		_items_store.remove (iter);
	}

	public void clear_list ()
	{
		_items.clear ();
		_items_store.clear ();
	}
	
	private void init_widgets (string label)
	{
		_items_label.xalign = 0;
		_items_label.label = label;
		
		_items_store = new ListStore (ItemsColumns.N_COLUMNS, typeof (string), typeof (string));
		_items_view = new ComboBox.with_model (_items_store);

		var renderer = new CellRendererText ();
		_items_view.pack_start (renderer, true);
		_items_view.add_attribute (renderer, "text", ItemsColumns.ITEM_LABEL);
		
		//Bind properties
		bind_property ("label", _items_label, "label", BindingFlags.BIDIRECTIONAL);
		
		//signals
		_items_view.changed.connect (() => notify_property ("current-item"));
	}
}
