/* main.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;
using I4uc.Core;
using I4uc.GtkFrontend;
using Gee;

//Constants
const double _PULSE_STEP = 0.25;
const OptionEntry[] _OPTIONS = {
	{ "version", 0, 0, OptionArg.NONE, ref _option_version, N_("Display version number"), null },
	{ "", 0, 0, OptionArg.FILENAME_ARRAY, ref _option_files, null, N_("[FILE...]") },
	{ null }
};

//Command line options
bool _option_version;
[CCode (array_length = false, array_null_terminated = true)]
string[] _option_files;

//Variables for splash screen 
SplashScreen _splash_screen;

// Application (creates views and presenters)
I4uc.GtkFrontend.App _app;

int main (string[] args)
{
	//I18n
	Intl.bindtextdomain (Config.GETTEXT_PACKAGE, Config.LOCALE_DIR);
	Intl.bind_textdomain_codeset (Config.GETTEXT_PACKAGE, "UTF-8");
	
	//Command line options
	try
	{
		var context = new OptionContext (_("- IDE for microcontrollers"));
		context.set_help_enabled (true);
		context.add_main_entries (_OPTIONS, Config.GETTEXT_PACKAGE);
		context.add_group (get_option_group (false));
		context.parse (ref args);
	}
	catch (GLib.OptionError e)
	{
		stderr.printf (e.message + "\n");
		stderr.printf (_("Run '%s --help' to see a full list of available command line options.\n"), args[0]);
		return 1;
	}
	
	if (_option_version)
	{
		print ("i4uc %s\n", Config.VERSION);
		return 0;
	}
	
	Gtk.init (ref args);
	
	_splash_screen = new SplashScreen ();
	_splash_screen.show_all ();
	
	start_loading.begin ();
	
	Gtk.main ();
	
	try
	{
		save_settings ();
	}
	catch (GLib.Error e)
	{
		warning (e.message);
	}
	return 0;
}

async void start_loading ()
{
	Idle.add (start_loading.callback);
	yield;
	_app = new I4uc.GtkFrontend.App ();
	try
	{
		//Load settings
		_splash_screen.text = _("Loading settings...");
		load_settings ();
		_splash_screen.fraction += _PULSE_STEP;
		Idle.add (start_loading.callback);
		yield;
		
		//Load builders
		_splash_screen.text = _("Loading builders...");
		load_builders ();
		_splash_screen.fraction += _PULSE_STEP;
		Idle.add (start_loading.callback);
		yield;
		
		//Load programmers
		_splash_screen.text = _("Loading programmers...");
		load_programmers ();
		_splash_screen.fraction += _PULSE_STEP;
		Idle.add (start_loading.callback);
		yield;
		
		//Load images
		_splash_screen.text = _("Loading images...");
		load_stock_items ();
		_splash_screen.fraction += _PULSE_STEP;
		Idle.add (start_loading.callback);
		yield;
		
		//Load user interface
		load_gui ();
		_splash_screen.hide ();
		_splash_screen.destroy ();
		_splash_screen = null;
		Idle.add (start_loading.callback);
		yield;
		
		//Open documents
		foreach (var file in _option_files)
		{
			var uri = File.new_for_path (file).get_uri ();
			_app.documents_logic.open_document (uri);
		}
	}
	catch (GLib.Error e)
	{
		warning (e.message);
	}
}

void load_gui ()
{
	_app.load_all ();
	
	_app.main_window_view.exit_clicked.connect (() => {
		if (!_app.documents_logic.close_all ())
			return true;
		_app.projects_logic.close_all_projects ();
		return false;
	});
}

/*
 * Load core and printer settings from ~/.i4uc/{i4ucrc,printersettings}
 */
void load_settings () throws GLib.Error, GLib.KeyFileError
{
	var settings_folder = Path.build_filename (Environment.get_home_dir (), ".i4uc");
	if (!FileUtils.test (settings_folder, FileTest.EXISTS))
		DirUtils.create (settings_folder, 0755);
	
	// Load core settings
	var settings_filename = Path.build_filename (settings_folder, "settings.xml");
	if (FileUtils.test (settings_filename, FileTest.EXISTS))
		SettingsLoader.load (_app.settings, settings_filename);
	
	// Load printer settings
	var key_file = new KeyFile ();
	
	var printer_settings_filename = Path.build_filename (settings_folder, "printersettings");
	if (!FileUtils.test (printer_settings_filename, FileTest.EXISTS) ||
	    !key_file.load_from_file (printer_settings_filename, KeyFileFlags.NONE))
		return;
	
	if (key_file.has_group ("Page setup settings"))
		_app.page_setup = new PageSetup.from_key_file (key_file, "Page setup settings");
	
	if (key_file.has_group ("Print settings"))
		_app.print_settings = new PrintSettings.from_key_file (key_file, "Print settings");
}

/*
 * Save core and printer settings to ~/.i4uc/{i4ucrc,printersettings}
 */
void save_settings () throws GLib.Error, GLib.KeyFileError
{
	// Save core settings
	var settings_str = _app.settings.to_xml ();
	var settings_filename = Path.build_filename (Environment.get_home_dir (), ".i4uc", "settings.xml");
	FileUtils.set_contents (settings_filename, settings_str);
	
	// Save printer settings
	var key_file = new KeyFile ();
	
	_app.page_setup.to_key_file (key_file, "Page setup settings");
	_app.print_settings.to_key_file (key_file, "Print settings");
	
	size_t len;
	var str = key_file.to_data (out len);
	var filename = Path.build_filename (Environment.get_home_dir (), ".i4uc", "printersettings");
	FileUtils.set_contents (filename, str, (long)len);
}

void load_builders () throws GLib.Error, GLib.KeyFileError
{
	// Get builders files in $(pkgdatadir)/builders/
	var builders_list = new ArrayList<File> ();
	builders_list.add_all (get_files (File.new_for_path (Config.BUILDERS_DIR), ".xml"));
	
	// Get builders files in $(HOME)/.i4uc/builders/. If the folder doesn't exists, create it
	var home_folder = File.new_for_path (Path.build_filename (Environment.get_home_dir (), ".i4uc", "builders"));
	builders_list.add_all (get_files (home_folder, ".xml", true));
	
	// Load builders
	foreach (var builder_file in builders_list)
	{
		var builder = new I4uc.Core.Builder ();
		try
		{
			BuilderLoader.load (builder, builder_file.get_uri ());
			_app.builders[builder.name] = builder;
		}
		catch (GLib.KeyFileError e)
		{
			warning (builder_file.get_path () + ": " + e.message);
			continue;
		}
	}
}

void load_programmers () throws GLib.Error, GLib.KeyFileError
{
	// Get programmers files in $(pkgdatadir)/programmers/
	var programmers_list = new ArrayList<File> ();
	programmers_list.add_all (get_files (File.new_for_path (Config.PROGRAMMERS_DIR), ".xml"));
	
	// Get programmers files in $(HOME)/.i4uc/programmers/. If the folder doesn't exists, create it
	var home_folder = File.new_for_path (Path.build_filename (Environment.get_home_dir (), ".i4uc", "programmers"));
	programmers_list.add_all (get_files (home_folder, ".xml", true));
	
	// Load programmers
	foreach (var programmer_file in programmers_list)
	{
		var programmer = new Programmer ();
		try
		{
			ProgrammerLoader.load (programmer, programmer_file.get_uri ());
			_app.programmers[programmer.name] = programmer;
		}
		catch (KeyFileError e)
		{
			warning (programmer_file.get_path () + ": " + e.message);
			continue;
		}
	}
}

/*
	Load i4uc stock items located in $(pkgdatadir)/pixmaps
*/
void load_stock_items () throws GLib.Error
{
	var icon_factory = new IconFactory ();
	
	var icon_set = new IconSet.from_pixbuf (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PIXMAPS_DIR, "programmer-icon.png")));
	icon_factory.add ("i4uc-programmer-icon", icon_set);
	
	icon_set = new IconSet.from_pixbuf (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PIXMAPS_DIR, "write-flash.png")));
	icon_factory.add ("i4uc-write-flash", icon_set);
	
	icon_set = new IconSet.from_pixbuf (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PIXMAPS_DIR, "read-flash.png")));
	icon_factory.add ("i4uc-read-flash", icon_set);
	
	icon_set = new IconSet.from_pixbuf (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PIXMAPS_DIR, "verify-flash.png")));
	icon_factory.add ("i4uc-verify-flash", icon_set);
	
	icon_set = new IconSet.from_pixbuf (new Gdk.Pixbuf.from_file (Path.build_filename (Config.PIXMAPS_DIR, "erase-flash.png")));
	icon_factory.add ("i4uc-erase-flash", icon_set);
	
	icon_factory.add_default ();
}

/*
 * Get the list of files in a folder according to an extension, create the folder if
 * dosn't exists
 */
Gee.List<File> get_files (File folder, string extension, bool create_folder=false) throws GLib.Error
{
	// Check if the folder exists. If not, create it if create_folder is true
	if (!folder.query_exists (null))
	{
		if (create_folder)
			folder.make_directory (null);
		return Gee.List.empty<File> ();
	}

	var files = new ArrayList<File> ();
	
	// Load files list with files that have the extension
	var enumerator = folder.enumerate_children (FILE_ATTRIBUTE_STANDARD_NAME, FileQueryInfoFlags.NONE, null);
	FileInfo file_info;
	while ((file_info = enumerator.next_file (null)) != null)
		if (file_info.get_name ().has_suffix (extension))
			files.add (folder.get_child (file_info.get_name ()));
	
	return files;
}
