/* searchbarview.vala
 *
 * Copyright (C) 2010  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;

public class I4uc.GtkFrontend.SearchBarView : VBox, I4uc.Core.SearchBarView
{
	private SourceView _source_view;
	private Entry _search_text_entry = new Entry ();
	private Button _search_backward_button = new Button ();
	private Button _search_forward_button = new Button ();
	private CheckMenuItem _highlight_all_check = new CheckMenuItem.with_label (_("Highlight all"));
	private CheckMenuItem _match_case_check = new CheckMenuItem.with_label (_("Match case"));
	private Entry _replace_text_entry = new Entry ();
	private Button _replace_button = new Button.with_label (_("Replace"));
	private Button _replace_all_button = new Button.with_label (_("Replace all"));
	private Menu _options_menu = new Menu ();
	
	public string search_text { set; get; }
	public bool search_backward_sensitive { set; get; }
	public bool search_forward_sensitive { set; get; }
	public bool highlight_all { set; get; }
	public bool match_case { set; get; }
	public string replace_text { set; get; }
	public bool replace_sensitive { set; get; }
	public bool replace_all_sensitive { set; get; }
	
	public SearchBarView (SourceView source_view)
	{
		_source_view = source_view;
		
		var highlight_tag = new TextTag ("highlight");
		highlight_tag.background = "#ffff00";
		_source_view.buffer.get_tag_table ().add (highlight_tag);
		
		//Search bar
		var close_button = Utils.create_close_button ();
		
		var search_label = new Label (_("Search for:"));
		
		_search_text_entry.primary_icon_stock = STOCK_PROPERTIES;
		_search_text_entry.secondary_icon_stock = STOCK_CLEAR;
		_search_text_entry.secondary_icon_tooltip_text = _("Clean search text");
		
		_search_backward_button.relief = ReliefStyle.NONE;
		_search_backward_button.focus_on_click = false;
		_search_backward_button.image = new Image.from_stock (STOCK_GO_BACK, IconSize.MENU);
		
		_search_forward_button.relief = ReliefStyle.NONE;
		_search_forward_button.focus_on_click = false;
		_search_forward_button.image = new Image.from_stock (STOCK_GO_FORWARD, IconSize.MENU);
		
		//Replace bar
		var replace_label = new Label (_("Replace with:"));
		
		_replace_text_entry.secondary_icon_stock = STOCK_CLEAR;
		_replace_text_entry.secondary_icon_tooltip_text = _("Clean replace text");
		
		_replace_button.relief = ReliefStyle.NONE;
		_replace_button.focus_on_click = false;
		_replace_button.image = new Image.from_stock (STOCK_FIND_AND_REPLACE, IconSize.MENU);
		
		_replace_all_button.relief = ReliefStyle.NONE;
		_replace_all_button.focus_on_click = false;
		
		var hbox = new HBox (false, 0);
		hbox.pack_start (close_button, false, false, 2);
		hbox.pack_start (search_label, false, false, 2);
		hbox.pack_start (_search_text_entry, false, false, 0);
		hbox.pack_start (_search_backward_button, false, false, 0);
		hbox.pack_start (_search_forward_button, false, false, 0);
		pack_start (hbox, true, true, 0);
		
		hbox = new HBox (false, 0);
		hbox.pack_start (new Label ("     "), false, false, 0);
		hbox.pack_start (replace_label, false, false, 2);
		hbox.pack_start (_replace_text_entry, false, false, 0);
		hbox.pack_start (_replace_button, false, false, 0);
		hbox.pack_start (_replace_all_button, false, false, 0);
		
		var replace_expander = new Expander ("<b>" + _("Replace") + "</b>");
		replace_expander.set_use_markup (true);
		replace_expander.add (hbox);
		pack_start (replace_expander, true, true, 0);
		
		//options menu
		_options_menu.append (_highlight_all_check);
		_options_menu.append (_match_case_check);
		_options_menu.show_all ();
		
		//Bind properties
		bind_property ("search-text", _search_text_entry, "text", BindingFlags.BIDIRECTIONAL);
		bind_property ("search-backward-sensitive", _search_backward_button, "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("search-forward-sensitive", _search_forward_button, "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("highlight-all", _highlight_all_check, "active", BindingFlags.BIDIRECTIONAL);
		bind_property ("match-case", _match_case_check, "active", BindingFlags.BIDIRECTIONAL);
		bind_property ("replace-text", _replace_text_entry, "text", BindingFlags.BIDIRECTIONAL);
		bind_property ("replace-sensitive", _replace_button, "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("replace-all-sensitive", _replace_all_button, "sensitive", BindingFlags.BIDIRECTIONAL);
		
		//connect signals
		_search_text_entry.icon_press.connect (on_search_text_entry_icon_press);
		_search_backward_button.clicked.connect (() => this.search_backward_clicked ());
		_search_forward_button.clicked.connect (() => this.search_forward_clicked ());
		_replace_text_entry.icon_press.connect (on_replace_text_entry_icon_press);
		_replace_button.clicked.connect (() => this.replace_clicked ());
		_replace_all_button.clicked.connect (() => this.replace_all_clicked ());
		close_button.clicked.connect (() => this.close_clicked ());
	}
	
	public void search (bool forward)
	{
		TextIter start;
		TextIter end;
		
		_source_view.buffer.get_selection_bounds (out start, out end);
		
		var flag = SourceSearchFlags.TEXT_ONLY | (this.match_case ? 0 : SourceSearchFlags.CASE_INSENSITIVE);
		
		bool found;
		if (forward)
			found = source_iter_forward_search (end, this.search_text, flag, out start, out end, null);
		else
			found = source_iter_backward_search (start, this.search_text, flag, out start, out end, null);
		
		if (found)
		{
			_source_view.buffer.place_cursor (start);
			_source_view.buffer.select_range (start, end);
			_source_view.scroll_to_iter (start, 0.25, true, 0.5, 0.5);
			_source_view.grab_focus ();
		}
		
		if (this.highlight_all)
		{
			remove_highlight ();
			apply_highlight ();
		}
		
		_source_view.grab_focus ();
	}
	
	public void apply_highlight ()
	{
		TextIter start;
		TextIter end;
		
		_source_view.buffer.get_start_iter (out start);
		
		var flag = SourceSearchFlags.TEXT_ONLY | (this.match_case ? 0 : SourceSearchFlags.CASE_INSENSITIVE);
		
		while (source_iter_forward_search (start, this.search_text, flag, out start, out end, null))
		{
			_source_view.buffer.apply_tag_by_name ("highlight", start, end);
			start = end;
		}
	}
	
	public void remove_highlight ()
	{
		TextIter start;
		TextIter end;
		
		_source_view.buffer.get_start_iter (out start);
		_source_view.buffer.get_end_iter (out end);
		_source_view.buffer.remove_tag_by_name ("highlight", start, end);
	}
	
	public void replace ()
	{
		TextIter start;
		TextIter end;
		
		if (_source_view.buffer.get_selection_bounds (out start, out end))
		{
			var selection = _source_view.buffer.get_text (start, end, false);
			var search = this.search_text;
			
			if (!this.match_case)
			{
				selection = selection.casefold ();
				search = search.casefold ();
			}
			
			if (selection != search)
				return;
			_source_view.buffer.begin_user_action ();
			_source_view.buffer.delete_selection (false, true);
			_source_view.buffer.insert_at_cursor (this.replace_text, (int)this.replace_text.length);
			_source_view.buffer.end_user_action ();
		}
		
		_source_view.grab_focus ();
	}
	
	public void replace_all ()
	{
		TextIter start;
		TextIter end;
		
		_source_view.buffer.get_start_iter (out start);
		
		var flag = SourceSearchFlags.TEXT_ONLY | (this.match_case ? 0 : SourceSearchFlags.CASE_INSENSITIVE);
				
		_source_view.buffer.begin_user_action ();
		_source_view.buffer.create_mark ("replace_mark", start, false);
		while (source_iter_forward_search (start, this.search_text, flag, out start, out end, null))
		{
			_source_view.buffer.move_mark_by_name ("replace_mark", end);
			_source_view.buffer.delete (start, end);
			_source_view.buffer.insert (start, this.replace_text, (int)this.replace_text.length);
			_source_view.buffer.get_iter_at_mark (out start, _source_view.buffer.get_mark ("replace_mark"));
		}
		_source_view.buffer.delete_mark_by_name ("replace_mark");
		_source_view.buffer.end_user_action ();
		
		if (this.highlight_all)
			remove_highlight ();
		
		_source_view.grab_focus ();
	}
	
	public new void grab_focus ()
	{
		_search_text_entry.grab_focus ();
	}
	
	private void on_search_text_entry_icon_press (EntryIconPosition icon_pos, Gdk.Event event)
	{
		if (icon_pos == EntryIconPosition.PRIMARY)
			_options_menu.popup (null, null, null, event.button.button, event.button.time);
		else
			_search_text_entry.text = "";
	}
	
	private void on_replace_text_entry_icon_press (EntryIconPosition icon_pos, Gdk.Event event)
	{
		if (icon_pos == EntryIconPosition.SECONDARY)
			_replace_text_entry.text = "";
	}
}
