/* documentpagelogic.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */

public class I4uc.Core.DocumentPageLogic : GLib.Object
{
	private App _app;
	private DocumentPageView _view;
	private Document _document = new Document ();
	private SearchBarLogic _search_bar_logic;
	private bool _externally_modified_adviced;
	private TabImage _prev_tab_image;
	
	public DocumentPageView view { get { return _view; } }
	public bool is_new { set; get; }
	
	public bool can_save
	{
		get
		{
			if (_is_new)
				return true;
			return _document.can_write;
		}
	}
	
	public signal void selected ();
	public signal void refresh_actions ();
	public signal void remove_document ();
	
	public DocumentPageLogic (App app, DocumentPageView view)
	{
		_app = app;
		_view = view;
		_search_bar_logic = new SearchBarLogic (_view.search_bar_view);
		
		//configure view
		_view.font_name = _app.settings.font;
		_view.tab_width = _app.settings.tab_width;
		_view.auto_indent = _app.settings.auto_indent;
		_view.highlight_current_line = _app.settings.highlight_current_line;
		_view.insert_spaces_instead_of_tabs = _app.settings.insert_spaces_instead_of_tabs;
		_view.show_line_numbers = _app.settings.show_line_numbers;
		_view.highlight_matching_brackets = _app.settings.highlight_matching_brackets;
		_view.wrap_mode = _app.settings.wrap_mode;
		
		_view.error_message_bar.visible = false;
		_view.error_message_bar.image = TabImage.ERROR;
		_view.reload_message_bar.visible = false;
		_view.reload_message_bar.image = TabImage.WARNING;
		_view.save_message_bar.visible = false;
		_view.save_message_bar.image = TabImage.WARNING;
		_view.search_bar_view_visible = false;
		_view.content_modified = false;
		
		//connect view signals
		_view.error_message_bar.ok_clicked.connect (on_error_message_bar_ok_clicked);
		_view.reload_message_bar.ok_clicked.connect (on_reload_message_bar_ok_clicked);
		_view.reload_message_bar.cancel_clicked.connect (on_reload_message_bar_cancel_clicked);
		_view.save_message_bar.ok_clicked.connect (on_save_message_bar_ok_clicked);
		_view.save_message_bar.cancel_clicked.connect (on_save_message_bar_cancel_clicked);
		_view.close_clicked.connect (on_close_clicked);
		_view.search_bar_view.close_clicked.connect (on_search_bar_close_clicked);
		_view.selected.connect (on_selected);
		_view.content_changed.connect (on_content_changed);
		_view.modified_changed.connect (on_modified_changed);
		_view.selection_changed.connect (() => this.refresh_actions ());
		_view.focused_in.connect (on_focused_in);
	}
	
	public void open (string uri)
	{
		var file = File.new_for_uri (uri);
		try
		{
			_view.title = uri;
			var file_info = file.query_info (FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, FileQueryInfoFlags.NONE, null);
			_view.tab_title = file_info.get_display_name ();
			var contents = _document.load_contents (uri);
			_view.tab_tooltip = "<b>%s</b> %s".printf (_("Name:"), _document.parse_name);
			_view.content = contents;
			_view.content_modified = false;
			_view.tab_mark = false;
			_app.add_recent (uri);
			_view.guess_language_id (_document.content_type);
			_externally_modified_adviced = false;
		}
		catch (GLib.Error e)
		{
			_view.content_visible = false;
			_view.error_message_bar.primary_text = _("Error trying to open ") + file.get_path ();
			_view.error_message_bar.secondary_text = e.message;
			_view.tab_image = TabImage.ERROR;
			_view.error_message_bar.visible = true;
		}
	}
	
	public void save (bool forced=false)
	{
		if (_is_new)
			save_as ();
		else if (_view.content_modified)
		{
			try
			{
				_document.save_contents (_view.content, null, forced);
				_view.content_modified = false;
				_view.tab_mark = false;
				_app.add_recent (_document.uri);
				_externally_modified_adviced = false;
			}
			catch (GLib.Error e)
			{
				if (e is IOError.WRONG_ETAG)
				{
					_view.save_message_bar.primary_text = _("The document was externally changed");
					_view.save_message_bar.secondary_text = _("Do you want to save the document?");
					_prev_tab_image = _view.tab_image;
					_view.tab_image = TabImage.WARNING;
					_view.save_message_bar.visible = true;
				}
				else
				{
					_view.error_message_bar.primary_text = _("Error trying to save ") + _document.path;
					_view.error_message_bar.secondary_text = e.message;
					_prev_tab_image = _view.tab_image;
					_view.tab_image = TabImage.ERROR;
					_view.error_message_bar.visible = true;
				}
			}
		}
	}
	
	public void save_as ()
	{
		string document_uri;
		var folder_uri = _app.settings.working_folder;
		var response = _app.view_factory.show_save_dialog (_view.tab_title, ref folder_uri, out document_uri);
		if (response == DialogResponse.OK)
		{
			try
			{
				_document.save_contents (_view.content, document_uri);
				_view.title = document_uri;
				_is_new = false;
				_view.tab_title = _document.display_name;
				_view.tab_tooltip = "<b>%s</b> %s".printf (_("Name:"), _document.parse_name);
				_view.tab_image = TabImage.FILE;
				_view.content_modified = false;
				_view.tab_mark = false;
				_app.add_recent (_document.uri);
				_app.settings.working_folder = folder_uri;
				_externally_modified_adviced = false;
			}
			catch (GLib.Error e)
			{
				_view.error_message_bar.primary_text = _("Error trying to save as ") + File.new_for_uri (document_uri).get_path ();
				_view.error_message_bar.secondary_text = e.message;
				_prev_tab_image = _view.tab_image;
				_view.tab_image = TabImage.ERROR;
				_view.error_message_bar.visible = true;
			}
		}
	}
	
	public void print ()
	{
		try
		{
			_view.show_print_dialog ();
		}
		catch (GLib.Error e)
		{
			_view.error_message_bar.primary_text = _("Error trying to print ") + _document.uri;
			_view.error_message_bar.secondary_text = e.message;
			_prev_tab_image = _view.tab_image;
			_view.tab_image = TabImage.ERROR;
			_view.error_message_bar.visible = true;
		}
	}
	
	public void undo ()
	{
		_view.undo ();
		this.refresh_actions ();
	}
	
	public void redo ()
	{
		_view.redo ();
		this.refresh_actions ();
	}
	
	public void cut ()
	{
		_view.cut ();
		this.refresh_actions ();
	}
	
	public void copy ()
	{
		_view.copy ();
		this.refresh_actions ();
	}
	
	public void paste ()
	{
		_view.paste ();
		this.refresh_actions ();
	}
	
	public void show_search_bar ()
	{
		_view.search_bar_view_visible = true;
		if (_view.search_bar_view.search_text != null)
			_view.search_bar_view.notify_property ("search-text");
		_view.search_bar_view.grab_focus ();
	}
	
	private void on_error_message_bar_ok_clicked ()
	{
		//Error produced trying to open a document
		if (!_view.content_visible)
			this.remove_document ();
		_view.tab_image = _prev_tab_image;
		_view.error_message_bar.visible = false;
		_view.grab_focus ();
	}
	
	private void on_reload_message_bar_ok_clicked ()
	{
		_view.tab_image = _prev_tab_image;
		_view.reload_message_bar.visible = false;
		open (_document.uri);
		_view.grab_focus ();
	}
	
	private void on_reload_message_bar_cancel_clicked ()
	{
		_view.tab_image = _prev_tab_image;
		_view.reload_message_bar.visible = false;
		_externally_modified_adviced = true;
		_view.grab_focus ();
	}
	
	private void on_save_message_bar_ok_clicked ()
	{
		_view.tab_image = _prev_tab_image;
		_view.save_message_bar.visible = false;
		save (true);
		_view.grab_focus ();
	}
	
	private void on_save_message_bar_cancel_clicked ()
	{
		_view.tab_image = _prev_tab_image;
		_view.save_message_bar.visible = false;
		_view.grab_focus ();
	}
	
	private void on_selected ()
	{
		if (!_is_new)
			_app.settings.working_folder = _document.folder;
		this.selected ();
	}
	
	private void on_content_changed ()
	{
		if (_view.search_bar_view_visible)
			_view.search_bar_view.notify_property ("search-text");
	}
	
	private void on_modified_changed ()
	{
		_view.tab_mark = _view.content_modified;
		this.refresh_actions ();
	}
	
	private void on_focused_in ()
	{
		if (_view.reload_message_bar.visible || _externally_modified_adviced)
			return;
		
		if (!_document.get_modified ())
			return;
		
		_view.reload_message_bar.primary_text = _("The file %s was externally modified").printf (_document.path);
		_view.reload_message_bar.secondary_text = _("Do you want to reload the file?");
		_prev_tab_image = _view.tab_image;
		_view.tab_image = TabImage.WARNING;
		_view.reload_message_bar.visible = true;
	}
	
	private void on_close_clicked ()
	{
		if (_view.content_modified)
		{
			var response = _view.show_close_message ();
			
			if (response == DialogResponse.CANCEL)
				return;
			
			if (response == DialogResponse.YES)
				save ();
		}
		this.remove_document ();
	}
	
	private void on_search_bar_close_clicked ()
	{
		_view.search_bar_view.remove_highlight ();
		_view.search_bar_view_visible = false;
		_view.grab_focus ();
	}
}
