/* optionslogic.vala
 *
 * Copyright (C) 2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public errordomain I4uc.Core.OptionError
{
	OPTION_VALUE_MISSING
}

public class I4uc.Core.OptionsLogic : GLib.Object
{
	private OptionsView _view;
	private Gee.List<Option> _options;
	private ArrayList<OptionView> _option_views = new ArrayList<OptionView> ();
	private Gee.Map<string, OptionSettings> _option_settings;
	private SerialPortUtils _serial_port_utils = create_serial_port_utils ();
	private UDevUtils? _udev_utils = create_udev_utils ();
	
	public signal void option_value_changed ();
	
	public OptionsLogic (OptionsView view)
	{
		_view = view;
		_view.extra_options_visible = false;
	}
	
	public void add_options (Gee.List<Option> options, Gee.Map<string, OptionSettings> option_settings)
	{
		_options = options;
		_option_settings = option_settings;
		update_options ();
	}
	
	public void clear_options ()
	{
		_option_views.clear ();
		_view.clear_options ();
	}
	
	public void update_options ()
	{
		if (_options == null)
			return;
		if (_option_views.size > 0)
			clear_options ();
		_view.extra_options_visible = false;
		foreach (var option in _options)
		{
			var option_view = _view.add_option (option.name, option.locale_label, option.value_type, option.optional, option.most_used);
			
			if (!option.most_used)
				_view.extra_options_visible = true;
			
			// Add a detailed tooltip
			option_view.tooltip = "<b>" + _("Description:") + "</b> " + option.locale_description;
			if (option.depends.size > 0)
			{
				var depends = "";
				foreach (var depend in option.depends)
					depends += get_option (depend).locale_label + ", ";
				option_view.tooltip += "\n<b>" + _("Depends:") + "</b> " + depends[0:-2];
			}
			
			// Load lists and ranges
			switch (option.value_type)
			{
				case "List":
					(option_view as ListOptionView).add_items ((option as ListOption).items);
					(option_view as ListOptionView).value = (option as ListOption).default;
					break;
				case "SerialPort":
					(option_view as ListOptionView).add_items (_serial_port_utils.list_serial_ports ());
					break;
				case "BaudRate":
					(option_view as ListOptionView).add_items (_serial_port_utils.list_baud_rates ());
					break;
				case "Range":
					var range_option = (option as RangeOption);
					(option_view as RangeOptionView).set_range (range_option.min, range_option.max, range_option.step);
					(option_view as RangeOptionView).value = range_option.default.to_string ();
					break;
				case "UDevList":
					if (_udev_utils != null)
					{
						var subsystem = (option as UDevOption).subsystem;
						var property_id = (option as UDevOption).property_id;
						var property_value = (option as UDevOption).property_value;
						(option_view as ListOptionView).add_items (_udev_utils.get_devices_file_list (subsystem, property_id, property_value));
					}
					break;
			}
			
			// Load saved options
			if (_option_settings.has_key (option.name))
			{
				option_view.active = _option_settings[option.name].active;
				option_view.value = _option_settings[option.name].value;
			}
			
			option_view.notify["value"].connect (on_option_value_changed);
			
			if (option.optional)
				option_view.notify["active"].connect (on_option_activated);
			
			_option_views.add (option_view);
		}
		on_option_activated ();
	}
	
	public static string get_options (Gee.List<Option> options, Gee.Map<string, OptionSettings> option_settings, Gee.List<string>? required_options, Gee.List<string>? optional_options, string? format=null) throws OptionError
	{
		// Fill required_options_str
		var required_options_str = "";
		if (required_options != null)
			foreach (var option_name in required_options)
			{
				var option_setting = option_settings[option_name];
				if (option_setting == null)
					continue;
				if (option_setting.active)
				{
					if (option_setting.value == null || option_setting.value == "")
						throw new OptionError.OPTION_VALUE_MISSING (_("Option value missing: ") + option_name);
					var option = get_option_from_list (options, option_name);
					var option_value = option_setting.value;
					if (option.value_type == "FilePath" || option.value_type == "FolderPath")
						option_value = Shell.quote (option_value);
					required_options_str += " " + option.format.replace ("%value", option_value);
				}
			}
		
		// Fill optional_options_str
		var optional_options_str = "";
		if (optional_options != null)
			foreach (var option_name in optional_options)
			{
				var option_setting = option_settings[option_name];
				if (option_setting == null)
					continue;
				if (option_setting.active)
				{
					if (option_setting.value == null || option_setting.value == "")
						throw new OptionError.OPTION_VALUE_MISSING (_("Option value missing: ") + option_name);
					var option = get_option_from_list (options, option_name);
					var option_value = option_setting.value;
					if (option.value_type == "FilePath" || option.value_type == "FolderPath")
						option_value = Shell.quote (option_value);
					optional_options_str += " " + option.format.replace ("%value", option_value);
				}
			}
		
		if (format == null)
			return @"$required_options_str $optional_options_str";
		var formatted_options = format;
		formatted_options = formatted_options.replace ("%required_options", required_options_str);
		formatted_options = formatted_options.replace ("%optional_options", optional_options_str);
		formatted_options = formatted_options.replace ("%options", @"$required_options_str $optional_options_str");
		return formatted_options;
	}
	
	public bool valid_required_options (Gee.List<string>? required_options)
	{
		// Return false if there's no options added
		if (_options == null)
			return false;
		
		// Return true if there's no required options
		if (required_options == null || required_options.size == 0)
			return true;
		
		// Check if the required options are valid
		var valid_options = true;
		foreach (var option_name in required_options)
		{
			var option_view = get_option_view (option_name);
			if (option_view == null)
				return false;
			valid_options &= option_view.value != null && option_view.value != "";
		}
		return valid_options;
	}
	
	private static Option? get_option_from_list (Gee.List<Option> options, string option_name)
	{
		foreach (var option in options)
			if (option.name == option_name)
				return option;
		return null;
	}
	
	private Option? get_option (string option_name)
	{
		return get_option_from_list (_options, option_name);
	}
	
	private OptionView? get_option_view (string option_name)
	{
		foreach (var option_view in _option_views)
			if (option_view.name == option_name)
				return option_view;
		return null;
	}
	
	private void on_option_value_changed ()
	{
		foreach (var option_view in _option_views)
		{
			// Save option
			if (!(option_view.name in _option_settings.keys))
			{
				_option_settings[option_view.name] = new OptionSettings ();
				_option_settings[option_view.name].name = option_view.name;
			}
			_option_settings[option_view.name].active = option_view.active;
			_option_settings[option_view.name].value = option_view.value ?? "";
		}
		this.option_value_changed ();
	}
	
	private void on_option_activated ()
	{
		foreach (var option in _options)
		{
			// Enable/Disable option sensitive
			var sensitive = true;
			foreach (var depend in option.depends)
				sensitive &= get_option_view (depend).active;
			get_option_view (option.name).sensitive = sensitive;
		}
		
		foreach (var option_view in _option_views)
		{
			// Enable/Disable value sensitive
			if (get_option (option_view.name).optional)
				option_view.value_sensitive = option_view.active;
			else
				option_view.active = true;
			
			// Save option
			if (!_option_settings.has_key (option_view.name))
			{
				_option_settings[option_view.name] = new OptionSettings ();
				_option_settings[option_view.name].name = option_view.name;
			}
			_option_settings[option_view.name].active = option_view.active;
			_option_settings[option_view.name].value = option_view.value ?? "";
		}
		this.option_value_changed ();
	}
}
