/* programmerloader.vala
 *
 * Copyright (C) 2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

namespace I4uc.Core.ProgrammerLoader
{
	private Programmer _programmer;
	private Option _new_option;
	private string _lang;
	private string _action;
	private bool _is_programmer;
	
	public void load (Programmer programmer, string uri) throws GLib.Error
	{
		_programmer = programmer;
		var file = File.new_for_uri (uri);
		string contents;
		file.load_contents (null, out contents);
		MarkupParser parser = { parser_start, parser_end, parser_text, null, null };
		MarkupParseContext context = new MarkupParseContext (parser, 0, null, null);
		context.parse (contents, -1);
	}
	
	private void parser_start (MarkupParseContext context, string name, string[] attr_names, string[] attr_values) throws MarkupError
	{
		switch (name)
		{
			case "programmer":
				_is_programmer = true;
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							_programmer.name = attr_values[i];
							break;
						case "firmware_types":
							foreach (var firmware_type in attr_values[i].split (","))
								_programmer.firmware_types.add (firmware_type);
							break;
					}
				}
				break;
			
			case "write_flash":
			case "read_flash":
			case "verify_flash":
			case "erase_flash":
			case "write_fuses":
			case "read_fuses":
			case "get_version":
				_action = name;
				break;
			
			case "option":
				string option_name="", format="", value_type="";
				bool optional=true, most_used=true;
				_is_programmer = false;
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							option_name = attr_values[i];
							break;
						case "format":
							format = attr_values[i];
							break;
						case "value_type":
							value_type = attr_values[i];
							break;
						case "optional":
							optional = attr_values[i].to_bool ();
							break;
						case "most_used":
							most_used = attr_values[i].to_bool ();
							break;
					}
				}
				if (value_type == "List")
					_new_option = new ListOption ();
				else if (value_type == "Range")
					_new_option = new RangeOption ();
				else if (value_type == "UDevList")
					_new_option = new UDevOption ();
				else
					_new_option = new Option ();
				_new_option.name = option_name;
				_new_option.format = format;
				_new_option.value_type = value_type;
				_new_option.optional = optional;
				_new_option.most_used = most_used;
				_programmer.options.add (_new_option);
				break;
			
			case "label":
			case "description":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "lang":
							_lang = attr_values[i];
							break;
					}
				}
				break;
				
			case "items":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "default":
							(_new_option as ListOption).default = attr_values[i];
							break;
					}
				}
				break;
			
			case "range":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "min":
							(_new_option as RangeOption).min = attr_values[i].to_int ();
							break;
						case "max":
							(_new_option as RangeOption).max = attr_values[i].to_int ();
							break;
						case "step":
							(_new_option as RangeOption).step = attr_values[i].to_int ();
							break;
						case "default":
							(_new_option as RangeOption).default = attr_values[i].to_int ();
							break;
					}
				}
				break;
			
			case "udev_items":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "subsystem":
							(_new_option as UDevOption).subsystem = attr_values[i];
							break;
						case "property_id":
							(_new_option as UDevOption).property_id = attr_values[i];
							break;
						case "property_value":
							(_new_option as UDevOption).property_value = attr_values[i];
							break;
					}
				}
				break;
		}
	}
	
	private void parser_end (MarkupParseContext context, string name) throws MarkupError
	{
		switch (name)
		{
			case "option":
				_is_programmer = true;
				break;
		}
	}
	
	private void parser_text (MarkupParseContext context, string text, size_t text_len) throws MarkupError
	{
		switch (context.get_element ())
		{
			case "command":
				switch (_action)
				{
					case "write_flash":
						_programmer.write_flash_command = text;
						_programmer.can_write_flash = text != "";
						break;
					
					case "read_flash":
						_programmer.read_flash_command = text;
						_programmer.can_read_flash = text != "";
						break;
					
					case "verify_flash":
						_programmer.verify_flash_command = text;
						_programmer.can_verify_flash = text != "";
						break;
					
					case "erase_flash":
						_programmer.erase_flash_command = text;
						_programmer.can_erase_flash = text != "";
						break;
					
					case "write_fuses":
						_programmer.write_fuses_command = text;
						_programmer.can_write_fuses = text != "";
						break;
					
					case "read_fuses":
						_programmer.read_fuses_command = text;
						_programmer.can_read_fuses = text != "";
						break;
					
					case "get_version":
						_programmer.get_version_command = text;
						_programmer.can_get_version = text != "";
						break;
				}
				break;
			
			case "required_options":
				var required_options_list = new ArrayList<string> ();
				foreach (var option_name in text.split (","))
					required_options_list.add (option_name);
				_programmer.required_options[_action] = required_options_list;
				break;
			
			case "optional_options":
				var optional_options_list = new ArrayList<string> ();
				foreach (var option_name in text.split (","))
					optional_options_list.add (option_name);
				_programmer.optional_options[_action] = optional_options_list;
				break;
			
			case "label":
				_new_option.label[_lang] = text;
				break;
			
			case "description":
				if (_is_programmer)
					_programmer.description[_lang] = text;
				else
					_new_option.description[_lang] = text;
				break;
			
			case "depends":
				foreach (var depend in text.split(","))
					_new_option.depends.add (depend);
				break;
			
			case "items":
				foreach (var item in text.split(","))
					(_new_option as ListOption).items.add (item);
				break;
		}
    }
}
