/* projectslogic.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public class I4uc.Core.ProjectsLogic : DocumentOpener, GLib.Object
{
	private App _app;
	private ProjectsView _view;
	private DocumentsLogic _documents_logic;
	private ProgrammersLogic _programmers_logic;
	private ProjectsPageLogic _page_logic;
	private ProfileBuilder _profile_builder;

	public ProjectsLogic (App app)
	{
		_app = app;
		_view = _app.projects_view;
		_documents_logic = _app.documents_logic;
		_programmers_logic = _app.programmers_logic;
		
		_documents_logic.add_document_opener (".i4uc", this);

		_page_logic = new ProjectsPageLogic (_app);
		_profile_builder = new ProfileBuilder (_app);

		//Configure view
		_view.add_file_sensitive = false;
		_view.remove_file_from_project_sensitive = false;
		_view.remove_file_from_profile_sensitive = false;
		_view.add_profile_sensitive = false;
		_view.edit_profile_sensitive = false;
		_view.page_view.profile_edit_sensitive = false;
		_view.duplicate_profile_sensitive = false;
		_view.remove_profile_sensitive = false;
		_view.build_profile_sensitive = false;
		_view.program_profile_sensitive = false;
		_view.stop_sensitive = false;
		_view.clean_profile_sensitive = false;
		_view.close_sensitive = false;
		_view.build_log_page.tab_title = _("Build process");
		_view.build_log_page.tab_image = TabImage.BUILDER;
		_view.build_log_page.message_format = "<b>%s</b>";
		_view.build_log_page.error_format = "<b><span color=\"#D00000\">%s</span></b>";
		_view.build_log_page.command_format = "\t<i>%s</i>";
		_view.build_log_page.output_format = "\t\t%s";

		//Connect view signals
		_view.new_clicked.connect (on_new_clicked);
		_view.add_file_clicked.connect (on_add_file_clicked);
		_view.remove_file_from_project_clicked.connect (on_remove_file_from_project_clicked);
		_view.remove_file_from_profile_clicked.connect (on_remove_file_from_profile_clicked);
		_view.add_profile_clicked.connect (on_add_profile_clicked);
		_view.edit_profile_clicked.connect (() => _page_logic.edit_current_profile ());
		_view.duplicate_profile_clicked.connect (() => _page_logic.duplicate_current_profile ());
		_view.remove_profile_clicked.connect (on_remove_profile_clicked);
		_view.build_profile_clicked.connect (on_build_profile_clicked);
		_view.program_profile_clicked.connect (on_program_profile_clicked);
		_view.stop_clicked.connect (() => _profile_builder.stop ());
		_view.clean_profile_clicked.connect (on_clean_profile_clicked);
		_view.close_clicked.connect (() => _page_logic.remove_project (_page_logic.current_project));
		_view.page_view.notify["current-project"].connect (on_project_changed);
		_view.page_view.notify["current-profile"].connect (on_profile_changed);
		_view.page_view.file_changed.connect (on_file_changed);
		_view.build_log_page.command_activated.connect (on_command_activated);
		_profile_builder.notify["is-building"].connect (on_profile_builder_is_building);
		_profile_builder.new_message.connect ((message) => _view.build_log_page.add_message (message));
		_profile_builder.new_error.connect ((error) => _view.build_log_page.add_error (error));
		_profile_builder.command_finished.connect ((command_context) => _view.build_log_page.add_command (command_context));
		_programmers_logic.notify["can-write-flash"].connect (on_programmers_logic_can_write_flash_changed);
	}

	public bool open_project (string project_uri)
	{
		var project = new Project ();
		try
		{
			project.open (project_uri);
			_page_logic.add_project (project);
			_app.add_recent (project_uri);
		}
		catch (GLib.Error e)
		{
			_app.view_factory.show_error_message (_("Error trying to open <<%s>>: %s").printf (project_uri, e.message));
			return false;
		}
		return true;
	}
	
	public bool open_document (string document_uri)
	{
		return open_project (document_uri);
	}

	public void close_all_projects ()
	{
		_page_logic.remove_all_projects ();
	}

	private void on_new_clicked ()
	{
		string project_uri;
		string project_name;
		if (_view.show_new_dialog (out project_uri, out project_name) == DialogResponse.OK)
		{
			var project = new Project ();
			project.name = project_name;

			var main_profile = new Profile ();
			main_profile.name = "main";
			project.profiles["main"] = main_profile;

			project.save (project_uri);
			_app.add_recent (project_uri);
			_page_logic.add_project (project);
		}
	}

	private void on_add_file_clicked ()
	{
		var dialog_view = _view.create_add_file_dialog_view ();
		var current_project = _page_logic.current_project;
		var dialog_logic = new AddFileDialogLogic (dialog_view, current_project);
		
		if (dialog_logic.run ())
		{
			string file;
			if (dialog_view.is_new)
			{
				// Create a new empty file
				var new_file = File.new_for_uri (current_project.get_file_uri (dialog_view.filename));
				var overwrite = true;
				if (new_file.query_exists (null))
				{
					// The file already exists, ask to create a new one
					var message = _("The file <<%s>> already exists, do you want to create a new one?").printf (new_file.get_path ());
					var response = _app.view_factory.show_yes_no_message (message, true);
	
					if (response == DialogResponse.CANCEL)
					{
						dialog_view.close ();
						return;
					}
	
					if (response == DialogResponse.NO)
						overwrite = false;
				}
				if (overwrite)
					new_file.replace_contents ("", 0, null, false, FileCreateFlags.NONE, null, null);
				file = dialog_view.filename;
			}
			else
			{
				// Import a file
				var import_file = File.new_for_uri (dialog_view.filename);
				var file_info = import_file.query_info (FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, FileQueryInfoFlags.NONE, null);
				var import_filename = file_info.get_display_name ();
				if (current_project.folder_uri != import_file.get_parent ().get_uri ())
				{
					// The file is in another folder, ask to copy the file
					var message = _("The file <<%s>> is outside the project's folder, do you want to copy the file?").printf (import_file.get_path ());
					var response = _app.view_factory.show_yes_no_message (message, false);
					
					if (response == DialogResponse.YES)
					{
						// The file already exists, ask to overwrite the file
						var dest_file = File.new_for_uri (current_project.get_file_uri (import_filename));
						message = _("Do you want to overwrite the file <<%s>>?").printf (dest_file.get_path ());
						if (dest_file.query_exists (null) && _app.view_factory.show_yes_no_message (message, false) == DialogResponse.NO)
						{
							dialog_view.close ();
							return;
						}
						import_file.copy (dest_file, FileCopyFlags.OVERWRITE, null, null);
					}
				}
				file = import_filename;
			}
			
			// Add the file to the project and to the selected profiles and open the file
			if (!(file in current_project.files))
				current_project.files.add (file);
			foreach (var profile in dialog_view.activated_profiles)
				if (!(file in current_project.profiles[profile].files))
					current_project.profiles[profile].files.add (file);
			current_project.save ();
			_page_logic.update_files_list ();
			_documents_logic.open_document (current_project.get_file_uri (file));
		}
		dialog_view.close ();
	}

	private void on_remove_file_from_project_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_file = _view.page_view.current_file;
		var message = _("Do you want to remove the file <<%s>> from %s project?").printf (current_file, current_project.name);
		if (_app.view_factory.show_yes_no_message (message, false) != DialogResponse.YES)
			return;
		foreach (var profile in current_project.profiles.values)
			if (current_file in profile.files)
				profile.files.remove (current_file);
		current_project.files.remove (current_file);
		current_project.save ();
		_page_logic.update_files_list ();
	}

	private void on_remove_file_from_profile_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_profile = _view.page_view.current_profile;
		var current_file = _view.page_view.current_file;
		var message = _("Do you want to remove the file <<%s>> from %s profile?").printf (current_file, current_profile);
		if (_app.view_factory.show_yes_no_message (message, false) != DialogResponse.YES)
			return;
		current_project.profiles[current_profile].files.remove (current_file);
		current_project.save ();
		_page_logic.update_files_list ();
	}

	private void on_add_profile_clicked ()
	{
		var dialog_view = _view.create_add_profile_dialog_view ();
		var current_project = _page_logic.current_project;
		var dialog_logic = new AddProfileDialogLogic (dialog_view, current_project);
		if (dialog_logic.run ())
		{
			var profile = new Profile ();
			profile.name = dialog_view.profile_name;
			profile.is_public = dialog_view.is_public;
			
			if (!dialog_view.is_new)
			{
				var import_project = new Project ();
				try
				{
					import_project.open (dialog_view.project_uri);
				}
				catch (GLib.Error e)
				{
					_app.view_factory.show_error_message (_("Error trying to open %s: %s\n").printf (dialog_view.project_uri, e.message));
					dialog_view.close ();
					return;
				}
				var import_profile = import_project.profiles[dialog_view.profile_name];
				if (current_project.folder_uri != import_project.folder_uri)
				{
					// The projects are in diferent folders, ask to copy the files
					var message = _("The files are outside the project's folder, do you want to copy the files?");
					if (_app.view_factory.show_yes_no_message (message, false) == DialogResponse.NO)
					{
						dialog_view.close ();
						return;
					}
					
					// If some of the files are in the project's folder, ask to overwrite the file
					foreach (var file in import_profile.files)
					{
						var orig_file = File.new_for_uri (import_project.get_file_uri (file));
						var dest_file = File.new_for_uri (current_project.get_file_uri (file));
						message = _("Do you want to overwrite the file <<%s>>?").printf (dest_file.get_path ());
						if (dest_file.query_exists (null) && _app.view_factory.show_yes_no_message (message, false) == DialogResponse.NO)
							continue;
						orig_file.copy (dest_file, FileCopyFlags.OVERWRITE, null, null);
						current_project.files.add (file);
						profile.files.add (file);
					}
				}
				else
				{
					// The projects are in the same folder, just add them
					foreach (var file in import_profile.files)
					{
						current_project.files.add (file);
						profile.files.add (file);
					}
				}
				
				// Copy build options
				profile.builder = import_profile.builder;
				profile.output_type = import_profile.output_type;
				profile.device = import_profile.device;
				profile.set_link_options (import_profile.link_options);
				profile.set_compile_options (import_profile.compile_options);
			}
			
			// Add the new profile, save the project and update de profiles list
			current_project.profiles[dialog_view.profile_name] = profile;
			current_project.save ();
			_page_logic.update_profiles_list ();
		}
		dialog_view.close ();
	}
	
	private void on_remove_profile_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_profile = _view.page_view.current_profile;
		var message = _("Do you want to remove the profile <<%s>> from %s project?").printf (current_profile, current_project.name);
		if (_app.view_factory.show_yes_no_message (message, false) != DialogResponse.YES)
			return;
		current_project.profiles.unset (current_profile);
		current_project.save ();
		_page_logic.update_profiles_list ();
	}
	
	private void on_build_profile_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_profile = current_project.profiles[_view.page_view.current_profile];
		var message = _("The %s of the profile isn't selected");
		
		if (current_profile.builder == "")
		{
			_app.view_factory.show_error_message (message.printf (_("builder")));
			return;
		}
		if (current_profile.output_type == "")
		{
			_app.view_factory.show_error_message (message.printf (_("output type")));
			return;
		}
		if (current_profile.device == "")
		{
			_app.view_factory.show_error_message (message.printf (_("device")));
			return;
		}
		
		//Save profile's files before building
		var files_to_save = new ArrayList<string> ();
		foreach (var file in current_profile.files)
			files_to_save.add (current_project.get_file_uri (file));
		_documents_logic.save_documents (files_to_save);
		
		_profile_builder.project = current_project;
		_profile_builder.profile = current_profile.name;

		_view.show_build_log_page ();
		
		try
		{
			_profile_builder.build ();
		}
		catch (GLib.Error e)
		{
			_app.view_factory.show_error_message (_("Error trying to build profile %s: %s").printf (current_profile.name, e.message));
		}
	}
	
	private void on_program_profile_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_profile = current_project.profiles[_view.page_view.current_profile];
		var message = _("The %s of the profile isn't selected");
		
		if (current_profile.builder == "")
		{
			_app.view_factory.show_error_message (message.printf (_("builder")));
			return;
		}
		if (current_profile.output_type == "")
		{
			_app.view_factory.show_error_message (message.printf (_("output type")));
			return;
		}
		
		var builder = _app.builders[current_profile.builder];
		var output_type = builder.output_types[current_profile.output_type];
		var firmware = current_project.name + "-" + current_profile.name + output_type.output_type;
		var firmware_uri = current_project.get_file_uri (firmware.replace (" ", "_"));
		var firmware_file = File.new_for_uri (firmware_uri);
		
		if (!firmware_file.query_exists (null))
		{
			_app.view_factory.show_error_message (_("The file %s doesn't exists").printf (firmware_file.get_path ()));
			return;
		}
		_programmers_logic.write_flash (firmware_uri);
	}
	
	private void on_clean_profile_clicked ()
	{
		var current_project = _page_logic.current_project;
		var current_profile = current_project.profiles[_view.page_view.current_profile];
		var message = _("The %s of the profile isn't selected");
		
		if (current_profile.builder == "")
		{
			_app.view_factory.show_error_message (message.printf (_("builder")));
			return;
		}
		if (current_profile.output_type == "")
		{
			_app.view_factory.show_error_message (message.printf (_("output type")));
			return;
		}
		
		_profile_builder.project = current_project;
		_profile_builder.profile = current_profile.name;
		
		_view.show_build_log_page ();
		
		try
		{
			_profile_builder.clean ();
		}
		catch (GLib.Error e)
		{
			_app.view_factory.show_error_message (_("Error trying to clean profile %s: %s").printf (current_profile.name, e.message));
		}
	}
	
	private void on_project_changed ()
	{
		var project_selected = _page_logic.current_project != null;
		_view.add_file_sensitive = project_selected;
		_view.add_profile_sensitive = project_selected;
		_view.close_sensitive = project_selected;
	}

	private void on_profile_changed ()
	{
		var profile_selected = _view.page_view.current_profile != null &&
		                       _view.page_view.current_profile != "__NONE__";
		var file_selected = _view.page_view.current_file != null;
		_view.remove_file_from_profile_sensitive = profile_selected && file_selected;
		_view.edit_profile_sensitive = profile_selected;
		_view.page_view.profile_edit_sensitive = profile_selected;
		_view.duplicate_profile_sensitive = profile_selected;
		_view.remove_profile_sensitive = profile_selected;
		_view.build_profile_sensitive = profile_selected && !_profile_builder.is_building;
		_view.clean_profile_sensitive = profile_selected && !_profile_builder.is_building;
		_view.program_profile_sensitive = profile_selected && !_profile_builder.is_building && _programmers_logic.can_write_flash;
	}

	private void on_file_changed ()
	{
		var profile_selected = _view.page_view.current_profile != null &&
		                       _view.page_view.current_profile != "__NONE__";
		var file_selected = _view.page_view.current_file != null;
		_view.remove_file_from_project_sensitive = file_selected;
		_view.remove_file_from_profile_sensitive = profile_selected && file_selected;
	}
	
	private void on_command_activated (string output, CommandContext command_context)
	{
		if (command_context.output_regex == null ||
		    command_context.output_regex == "")
			return;
		
		try
		{
			var regex = new Regex (command_context.output_regex);
			
			MatchInfo match_info;
			if (!regex.match (output, 0, out match_info))
				return;
			
			var line = match_info.fetch_named ("line");
			_documents_logic.open_document (command_context.source_uri, line.to_int ());
		}
		catch (Error e)
		{
			warning (e.message);
		}
	}
	
	private void on_profile_builder_is_building ()
	{
		var is_building = _profile_builder.is_building;
		_view.build_profile_sensitive = !is_building;
		_view.stop_sensitive = is_building;
		_view.program_profile_sensitive = !is_building;
	}
	
	private void on_programmers_logic_can_write_flash_changed ()
	{
		var profile_selected = _view.page_view.current_profile != null &&
		                       _view.page_view.current_profile != "__NONE__";
		_view.program_profile_sensitive = profile_selected && !_profile_builder.is_building && _programmers_logic.can_write_flash;
	}
}
