/* projectspagelogic.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public class I4uc.Core.ProjectsPageLogic : GLib.Object
{
	private App _app;
	private ProjectsPageView _view;
	private DocumentsLogic _documents_logic;
	private ArrayList<Project> _projects = new ArrayList<Project> ();
	private Project _current_project;
	private string _current_profile;
	private OptionsLogic _compile_options_logic;
	private OptionsLogic _link_options_logic;

	public Project current_project
	{
		set { _view.current_project = value.uri; }
		get { return _current_project; }
	}
	
	public Gee.List<Project> projects { owned get { return _projects.read_only_view; } }
	
	public ProjectsPageLogic (App app)
	{
		_app = app;
		_view = _app.projects_view.page_view;
		_documents_logic = _app.documents_logic;
		_compile_options_logic = new OptionsLogic (_view.builders_list.compile_options_view);
		_link_options_logic = new OptionsLogic (_view.builders_list.link_options_view);
		
		//Configure view
		_view.tab_title = _("Projects");
		_view.profiles_check_list_sensitive = false;
		_view.profiles_check_list_visible = false;
		_view.builders_list_sensitive = false;
		_view.builders_list_visible = false;
		_view.builders_list.compile_options_visible = false;
		
		//Add sorted list of builders
		var builders = new ArrayList<string> ();
		builders.add_all (_app.builders.keys);
		builders.sort ();
		foreach (var builder in builders)
			_view.builders_list.add_builder (builder);

		//Connect view signals
		_view.notify["current-project"].connect (on_project_changed);
		_view.notify["current-profile"].connect (on_profile_changed);
		_view.profile_edit_clicked.connect (edit_current_profile);
		_view.profile_check_changed.connect (on_profile_check_changed);
		_view.file_activated.connect (on_file_activated);
		_view.file_changed.connect (on_file_changed);
		_view.builders_list.notify["current-builder"].connect (on_builder_changed);
		_view.builders_list.notify["current-output-type"].connect (on_output_type_changed);
		_view.builders_list.notify["current-device"].connect (on_device_changed);
		_compile_options_logic.option_value_changed.connect (on_compile_options_changed);
		_link_options_logic.option_value_changed.connect (on_link_options_changed);
	}

	public void add_project (Project project)
	{
		if (!(project in _projects))
		{
			_view.add_project (project.uri, project.name);
			_projects.add (project);
		}
		_view.current_project = project.uri;
	}

	public void remove_project (Project project)
	{
		if (!(project in _projects))
			return;
		_projects.remove (project);
		_view.remove_project (project.uri);
		if (_projects.size > 0)
			_view.current_project = _projects[0].uri;
	}

	public void remove_all_projects ()
	{
		for (int i = _projects.size-1; i >= 0; i--)
		{
			var project = _projects[i];
			_projects.remove (project);
			_view.remove_project (project.uri);
		}
	}

	public void update_profiles_list ()
	{
		on_project_changed ();
	}
	
	public void update_files_list ()
	{
		on_profile_changed ();
	}
	
	public void edit_current_profile ()
	{
		var dialog_view = _view.create_profile_dialog_view ();
		var profile = _current_project.profiles[_current_profile];
		var dialog_logic = new ProfileDialogLogic (dialog_view, _current_project, profile);
		
		if (dialog_logic.run ())
		{
			_current_project.profiles.unset (profile.name);
			profile.name = dialog_view.profile_name;
			profile.is_public = dialog_view.is_public;
			profile.files.clear ();
			profile.files.add_all (dialog_view.activated_files);
			_current_project.profiles[profile.name] = profile;
			_current_project.save ();
			_view.change_profile (_current_profile, profile.name);
		}
		dialog_view.close ();
	}
	
	public void duplicate_current_profile ()
	{
		var dialog_view = _view.create_profile_dialog_view ();
		var profile = _current_project.profiles[_current_profile];
		var dialog_logic = new ProfileDialogLogic (dialog_view, _current_project, profile, true);
		
		if (dialog_logic.run ())
		{
			var new_profile = new Profile ();
			new_profile.name = dialog_view.profile_name;
			new_profile.is_public = dialog_view.is_public;
			new_profile.builder = profile.builder;
			new_profile.output_type = profile.output_type;
			new_profile.device = profile.device;
			new_profile.set_link_options (profile.link_options);
			new_profile.set_compile_options (profile.compile_options);
			new_profile.files.add_all (dialog_view.activated_files);
			_current_project.profiles[new_profile.name] = new_profile;
			_current_project.save ();
			on_project_changed ();
			_current_profile = new_profile.name;
			_view.current_profile = _current_profile;
		}
		dialog_view.close ();
	}

	private void on_project_changed ()
	{
		_view.clear_profiles_list ();

		_current_project = get_project_from_uri (_view.current_project);
		if (_current_project == null)
			return;

		_view.current_project_tooltip = "<b>%s</b> %s\n".printf (_("Name:"), _current_project.name) +
		                                "<b>%s</b> %s".printf (_("File:"), _current_project.path);
		
		//Load profiles combo
		_view.add_profile ("__NONE__", _("None"), false);
		foreach (var profile in _current_project.profiles.values)
			_view.add_profile (profile.name);
		_view.current_profile = "__NONE__";
	}
	
	private void on_profile_changed ()
	{
		_view.clear_files_list ();
		_view.profiles_check_list_sensitive = false;
		_view.profiles_check_list_visible = false;
		_view.builders_list_sensitive = false;
		_view.builders_list_visible = false;
		_view.builders_list.current_builder = "";
		_view.builders_list.current_output_type = "";
		_view.builders_list.current_device = "";
		_compile_options_logic.clear_options ();
		_link_options_logic.clear_options ();

		if (_current_project == null)
			return;
		
		_current_profile = _view.current_profile;
		if (_current_profile == null)
			return;

		if (_current_profile == "__NONE__")
		{
			foreach (var file in _current_project.files)
				_view.add_file (file);
			_view.profiles_check_list_visible = true;
			return;
		}
		
		var profile = _current_project.profiles[_current_profile];
		
		foreach (var file in profile.files)
			_view.add_file (file);
		
		// Check if builder and device exists before using them
		if (_app.builders.has_key (profile.builder))
		{
			var builder = _app.builders[profile.builder];
			_view.builders_list.current_builder = profile.builder;
			
			if (builder.output_types.has_key (profile.output_type))
				_view.builders_list.current_output_type = profile.output_type;
			
			if (profile.device in builder.devices)
				_view.builders_list.current_device = profile.device;
			
			// Load link options
			if (builder.output_types.has_key (profile.output_type))
			{
				var output_type = builder.output_types[profile.output_type];
				if (!profile.link_options.has_key (output_type.name))
					profile.link_options[output_type.name] = new HashMap<string, OptionSettings> ();
				_link_options_logic.add_options (output_type.options, profile.link_options[output_type.name]);
			}
		}
		
		_view.builders_list_sensitive = true;
		_view.builders_list_visible = true;
	}

	private void on_profile_check_changed (string profile, bool active)
	{
		if (!_view.profiles_check_list_sensitive)
			return;

		if (_current_project == null || _view.current_file == null)
			return;

		if (active)
			_current_project.profiles[profile].files.add (_view.current_file);
		else
			_current_project.profiles[profile].files.remove (_view.current_file);
		_current_project.save ();
	}

	private void on_file_activated ()
	{
		var document_uri = _current_project.get_file_uri (_view.current_file);
		_documents_logic.open_document (document_uri);
	}

	private void on_file_changed ()
	{
		_view.profiles_check_list_sensitive = false;
		_view.uncheck_all_profiles ();
		_view.builders_list.compile_options_visible = false;

		if (_current_project == null || _view.current_file == null)
			return;
		
		// Check selected profiles
		foreach (var profile in _current_project.profiles.values)
			_view.check_profile (profile.name, _view.current_file in profile.files);
		_view.profiles_check_list_sensitive = true;
		
		// Show compile options if the file can be compiled
		if (_current_profile == null ||
		    _view.builders_list.current_builder == null ||
		    _view.builders_list.current_builder == "")
			return;
		
		var profile = _current_project.profiles[_current_profile];
		var builder = _app.builders[_view.builders_list.current_builder];
		
		var source_type = builder.get_source_type_from_filename (_view.current_file);
		if (source_type == null)
			return;
		
		// Load compile options
		if (!profile.compile_options.has_key (source_type.name))
			profile.compile_options[source_type.name] = new HashMap<string, OptionSettings> ();
		_compile_options_logic.add_options (source_type.options, profile.compile_options[source_type.name]);
		
		_view.builders_list.compile_options_visible = true;
	}

	private void on_builder_changed ()
	{
		_view.builders_list.clear_output_types_list ();
		_view.builders_list.clear_devices_list ();
		_compile_options_logic.clear_options ();
		_link_options_logic.clear_options ();

		if (_current_project == null || _current_profile == null)
			return;
		
		var profile = _current_project.profiles[_current_profile];
		var current_builder = _view.builders_list.current_builder;
		if (_view.builders_list_sensitive)
		{
			profile.builder = current_builder ?? "";
			profile.parameters_changed = true;
			_current_project.save ();
		}

		if (current_builder == null || current_builder == "")
			return;
		
		var builder = _app.builders[current_builder];
		
		// Load output types
		foreach (var output_type in builder.output_types.keys)
			_view.builders_list.add_output_type (output_type);
		
		// Load devices
		foreach (var device in builder.devices)
			_view.builders_list.add_device (device);
		
		// Show compile options if there's a file selected that can be compiled
		_view.builders_list.compile_options_visible = false;
		
		if (_view.current_file == null)
			return;
		
		var source_type = builder.get_source_type_from_filename (_view.current_file);
		if (source_type == null)
			return;
		
		// Load compile options
		if (!profile.compile_options.has_key (source_type.name))
			profile.compile_options[source_type.name] = new HashMap<string, OptionSettings> ();
		_compile_options_logic.add_options (source_type.options, profile.compile_options[source_type.name]);
		
		_view.builders_list.compile_options_visible = true;
	}

	private void on_output_type_changed ()
	{
		if (!_view.builders_list_sensitive)
			return;

		if (_current_project == null || _current_profile == null)
			return;
		
		var profile = _current_project.profiles[_current_profile];
		
		// Save output type change
		var current_output_type = _view.builders_list.current_output_type;
		profile.output_type = current_output_type ?? "";
		profile.parameters_changed = true;
		_current_project.save ();
		
		if (!_app.builders.has_key (profile.builder))
			return;
		var builder = _app.builders[profile.builder];
		
		// Load link options
		if (!builder.output_types.has_key (profile.output_type))
			return;
		var output_type = builder.output_types[profile.output_type];
		if (!profile.link_options.has_key (output_type.name))
			profile.link_options[output_type.name] = new HashMap<string, OptionSettings> ();
		_link_options_logic.add_options (output_type.options, profile.link_options[output_type.name]);
	}
	
	private void on_device_changed ()
	{
		if (!_view.builders_list_sensitive)
			return;

		if (_current_project == null || _current_profile == null)
			return;
		
		var current_device = _view.builders_list.current_device;
		_current_project.profiles[_current_profile].device = current_device ?? "";
		_current_project.profiles[_current_profile].parameters_changed = true;
		_current_project.save ();
	}
	
	private void on_compile_options_changed ()
	{
		if (!_view.builders_list_sensitive || !_view.builders_list.compile_options_visible)
			return;
		
		if (_current_project == null || _current_profile == null || _view.current_file == null)
			return;
		
		_current_project.profiles[_current_profile].parameters_changed = true;
		_current_project.save ();
	}
	
	private void on_link_options_changed ()
	{
		if (!_view.builders_list_sensitive)
			return;
		
		if (_current_project == null || _current_profile == null)
			return;
		
		_current_project.profiles[_current_profile].parameters_changed = true;
		_current_project.save ();
	}

	private Project? get_project_from_uri (string? project_uri)
	{
		foreach (var project in _projects)
			if (project.uri == project_uri)
				return project;
		return null;
	}
}
